
FUNCTION_BLOCK DAM_REF
	(*DAM_REF: Representation of the DAM for Danfoss Drive Servo with EtherCAT*)
	VAR_INPUT
		ModuleOk : BOOL; 						(*References ModuleOK from AutomationStudio*)
		NodeNumber : USINT; 					(* Node number of the device *)
		SlotName : STRING[80]; 				(*Name of the POWERLINK slot that this axis is physically connected to*)
		TPDO : ARRAY[0..8] OF UDINT; 		(* Data for IO Mapping *)
	END_VAR
	VAR_OUTPUT
		RPDO : ARRAY[0..8] OF UDINT; 		(* Data for IO Mapping *)
	END_VAR
	VAR
		intern: 						iDD_DAM_REF_INTERN_DAM; 	(* internal variable *)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_Power_DAM
	(*DD_Power: PLCopen FB for DAM:
	This function block controls the DC link voltage on the output line (not the FB itself)*)
	VAR_INPUT
		Enable : 			BOOL := FALSE;				(*As long as this input is true, DC link voltage is being enabled on the output line*)
		TimeLimit : 		TIME := t#0ms;				(*Time out after which an error is signalled, if the Status has not changed to true while Enable is true. 0 will disable the TimeLimit*)
	END_VAR
	VAR_OUTPUT
		Status : 			BOOL; 						(*Effective state of the DC link voltage on the output lines*)
		Valid : 			BOOL; 						(*If true, a valid set of outputs is available at the FB*)
		Error : 			BOOL; 						(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 			(*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Dam : 			DAM_REF;					(*Reference to the DAM*)
	END_VAR
	VAR
		intern : iDD_INTERN_POWER_DDS; (*internal variable*)
		pDamIntern : REFERENCE TO iDD_DAM_REF_INTERN_DAM; (*internal variable of DAM*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadDamError_DAM
	(*DD_ReadDamError: FB for DAM:
	This function block presents general DAM errors not relating to the function blocks
	(for instance DAM errors, voltage errors, communication errors).*)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 			(*Get the DAM information constantly while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		DamErrorID : 		WORD; 					(*The value of the DAM error*)
	END_VAR
	VAR_IN_OUT
		Dam : 			DAM_REF; 				(*Reference to the DAM*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_DAM; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadDamInfo_DAM
	(*DD_ReadDamInfo: FB for DAM:
	This function block reads information concerning a DAM, like modes and certain status information.*)
	VAR_INPUT
		Enable : 						BOOL := FALSE; 			(*Get the DAM information constantly while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : 						BOOL; 					(*A valid set of outputs is available at the FB*)
		Busy : 						BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 						BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 					DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		CommunicationReady : 		BOOL; 					(*"Network" is initialized and ready for communication*)
		ReadyForPowerOn : 			BOOL; 					(*DAM is ready to be enabled (power on)*)
		PowerOn : 					BOOL;					(*If TRUE shows that the power stage is switched ON*)
		ControlMode :					DD_CONTROL_MODE_DDS; (* Indicates the current control mode *)
		InputVoltageOk :				BOOL;					(* Indicates that the UDC is ok *)
		ChargeResistorProtection :		BOOL;					(* Indicates that the charge resistor protection is active and therefore the device cannot be enabled  *)
		UauxDisabled :				BOOL;					(* U aux is disabled on the outgoing hybrid line  *)
		DamWarning : 				BOOL; 					(*Warning(s) on the DAM is present*)
		DamError : 					BOOL;					(*Error(s) on the DAM is present*)
	END_VAR
	VAR_IN_OUT
		Dam : 						DAM_REF; 				(*Reference to the DAM*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadDamWarning_DAM
	(*DD_ReadDamWarning: FB for DAM:
	This function block presents general DAM warnings not relating to the function blocks *)
	VAR_INPUT
		Enable : 				BOOL := FALSE; 			(*Get the DAM information constantly while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid :				BOOL; 					(*A valid output is available at the FB*)
		Busy : 				BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 				BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 			DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		DamWarningID : 		WORD; 					(*The value of the DAM warning*)
	END_VAR
	VAR_IN_OUT
		Dam : 				DAM_REF; 				(*Reference to the DAM*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_DAM; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadDcLinkPower_DAM
	(*DD_ReadDcLinkPower: FB for DAM:
	This function block returns the value of the actual DC link power as long as "Enable" is set.*)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 				(*Get the value of the parameter continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid :			BOOL; 						(*A valid output is available at the FB*)
		Busy : 			BOOL; 						(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 						(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 			(*Error identification and instance identifier*)
		Power : 			REAL; 						(*The value of the actual DC link power [Ampere]*)
	END_VAR
	VAR_IN_OUT
		Dam : 			DAM_REF; 					(*Reference to the DAM*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadDcLinkVoltage_DAM
	(*DD_ReadDcLinkVoltage: FB for DAM:
	This function block returns the value of the actual DC link voltage as long as "Enable" is set.*)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 			(*Get the value of the parameter continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL;					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		Voltage : 		INT; 					(*The value of the actual DC link voltage [V]*)
	END_VAR
	VAR_IN_OUT
		Dam : 			DAM_REF; 				(*Reference to the DAM*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadParameter4_DAM
	(*DD_ReadParameter4: FB for DAM:
	FB to asynchronously read general objects of up to 4 bytes from the object dictionary of the DAM *)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 			(*Get the value of the parameter continuously while enabled*)
		Index : 			UINT := 0; 				(*Index of the object to be read*)
		Subindex : 		USINT := 0; 				(*Subindex of the object to be read*)
	END_VAR
	VAR_OUTPUT
		Valid : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		AbortCode : 		DWORD; 				(*SDO abort code in case of error*)
		Value : 			DWORD; 				(*Value of the specified parameter*)
	END_VAR
	VAR_IN_OUT
		Dam : 			DAM_REF; 				(*Reference to the DAM*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_DAM; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadPosGuideValueRef_DAM
	(*DD_ReadPosGuideValueRef: FB for DAM:
	This function block returns the value of the position guide value reference as long as "Enable" is set.*)
	VAR_INPUT
		Enable : 				BOOL := FALSE; 			(*Get the value of the parameter continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : 				BOOL; 					(*A valid output is available at the FB*)
		Busy : 				BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 				BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 			DD_ERROR_DDS;		(*Error identification and instance identifier*)
		PosGuideValue : 		UDINT; 					(*The value of the position guide value reference*)
	END_VAR
	VAR_IN_OUT
		Dam : 				DAM_REF; 				(*Reference to the DAM*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadVelGuideValueRef_DAM
	(*DD_ReadVelGuideValueRef: FB for DAM:
	This function block returns the value of the velicty guide value reference as long as "Enable" is set.*)
	VAR_INPUT
		Enable : 				BOOL := FALSE; 			(*Get the value of the parameter continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : 				BOOL; 					(*A valid output is available at the FB*)
		Busy : 				BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 				BOOL;					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 			DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		VelGuideValue : 		REAL; 					(*The value of the velocity guide value reference*)
	END_VAR
	VAR_IN_OUT
		Dam : 				DAM_REF; 				(*Reference to the DAM*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadVersion_DAM
	(*DD_ReadVersion: FB for DAM:
	FB to read the version information out f the DAM*)
	VAR_INPUT
		Execute : 			BOOL := FALSE; 			(*Read the information at rising edge*)
	END_VAR
	VAR_OUTPUT
		Done : 				BOOL; 					(*A valid set of outputs is available at the FB*)
		Busy : 				BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 				BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 			DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		SerialNumber : 		STRING[18]; 				(*Serial number of the DAM*)
		MajorVersionNo : 		UINT; 					(*Major firmware number*)
		MinorVersionNo : 		UINT; 					(*Minor firmware number*)
		PatchVersionNo : UINT; (*Patch firmware number*)
		PackageLabel : STRING[20]; (*Label of the software package*)
	END_VAR
	VAR_IN_OUT
		Dam : 				DAM_REF; 				(*Reference to the DAM*)
	END_VAR
	VAR
		intern : iDD_INTERN_READ_VERS_DAM; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_Reset_DAM
	(*DD_Reset: PLCopen FB for DAM:
	This function block makes the transition from the state "Error" to "Standby" by resetting all internal DAM-related errors - 
	it does not affect the output of the FB instances.*)
	VAR_INPUT
		Execute : 		BOOL := FALSE; 			(*Resets all internal DAM-related errors*)
	END_VAR
	VAR_OUTPUT
		Done : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL;					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Dam : 			DAM_REF; 				(*Reference to the DAM*)
	END_VAR
	VAR
		intern : iDD_INTERN_RESET_DDS; (*internal variable*)
		pDamIntern : REFERENCE TO iDD_DAM_REF_INTERN_DAM; (*internal variable of DAM*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_SetGuideValueRef_DAM
	(*DD_SetGuideValueRef: FB for DAM:
	This function block sets the position guide value reference object to the given value.
	The internally calculated offset will be stored permanently inside the DAM. 
	It is recommended that the guide value is constant during usage of this FB. 
	The value has to be given in guide value units.*)
	VAR_INPUT
		Execute : 		BOOL := FALSE;			(*Sets the position guide value reference object to the given value*)
		Position : 		UDINT := 0;				(*Value of the position guide value reference object that should be set*)
	END_VAR
	VAR_OUTPUT
		Done : 			BOOL;					(*The value has successfully been set in the DAM.*)
		Busy : 			BOOL;					(*The FB is not finished and new output values are to be expected.*)
		Error : 			BOOL;					(*Signals that an error has occurred within the FB.*)
		ErrorInfo : 		DD_ERROR_DDS;		(*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Dam : 			DAM_REF; 				(*Reference to the DAM*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_WRITE_DAM; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_SimulateGuideValue_DAM
	(*DD_SimulateGuideValue: FB for DAM:
	This function block instructs the DAM to do a simulation of a guide value with the given parameters.
	The guide value ref can be read with the FBs DD_ReadPosGuideValueRef_DAM and DD_ReadVelGuideValueRef_DAM*)
	VAR_INPUT
		Enable : 				BOOL := FALSE; 			(*As long as Enable is true, the guide value is simulated by the DAM. Enable = FALSE will stop the simulation immediately!*)
		Velocity :				REAL := 0.0;				(*Velocity of the guide value. Given in rps*)
		Acceleration :			REAL := 0.0;				(*Acceleration that is used for the guide value calculation. Given in rps*)
		Deceleration :			REAL := 0.0;				(*Deceleration that is used for guide value calculation. Given in rps*)
	END_VAR
	VAR_OUTPUT
		Valid : 				BOOL; 					(*A valid output is available at the FB*)
		Busy : 				BOOL;					(*The FB is not finished and new output values are to be expected*)
		Error : 				BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 			DD_ERROR_DDS; 		(*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Dam : 				DAM_REF; 				(*Reference to the DAM*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
		pDamIntern : REFERENCE TO iDD_DAM_REF_INTERN_DAM; (*internal variable of DAM*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_Trace_DAM
	(*DD_Trace: FB for DAM:
	This function block is used to start a trace in the DAM with the settings given in the input variables. After
	sending the settings and starting the trace, the FB will poll the DAM until the data has been recorded
	and afterwards uploads the data automatically.*)
	VAR_INPUT
		Execute : 				BOOL := FALSE; 									(*Starts the trace functionality at rising edge and keeps on polling*)
		Abort : 					BOOL := FALSE; 									(*Abort the on-going trace. Note that new values are only evaluated on a rising edge of Execute.*)
		pTraceBuffer : 			UDINT; 											(*Reference to a buffer where the read trace data shall be placed*)
		TraceBufferSize : 			UDINT := 0; 										(*Size of the trace buffer (size of the provided buffer given in Byte; use SIZEOF() function)*)
		SamplingRate : 			DD_SAMPLING_RATE_DDS := ddFastTask_DDS; 		(*Sampling rate of the trace*)
		SampleCount : 			UINT := 4000; 										(*Number of samples to be traced per channel*)
		SubSampling : 			UINT := 1; 										(*Multiplier to adjust time difference between trace samples*)
		SignalIDs : 				ARRAY[0..7] OF UDINT :=[ 8(0)]; 						(*Holds the IDs of the signals that should be traced*)
		TriggerID : 				UDINT := 0; 										(*ID of the signal used for triggering; Use 0 to have instantaneous tracing.*)
		TriggerPoint : 				USINT := 10; 										(*Holds the information on how many pre-trigger history is traced [in percentage]*)
		TriggerLevel : 			REAL := 0.0; 										(*Level upon which will be triggered*)
		TriggerSlope : 			BOOL := TRUE; 									(*TRUE: will trigger on rising slope; FALSE: will trigger on falling slope*)
	END_VAR
	VAR_OUTPUT
		Done : 					BOOL; 											(*The trace has successfully been recorded and read from the device.*)
		Busy : 					BOOL; 											(*The FB is not finished and new output values are to be expected*)
		CommandAborted : 		BOOL; 											(*Trace has been aborted.*)
		Error : 					BOOL; 											(*Signals that an error has occurred within the FB*)
		ErrorInfo : 				DD_ERROR_DDS;								(*Error identification and instance identifier*)
		Status : 					USINT; 											(*Holds the status of the tracing:
																				 1= Trace configured and started
																				 2= Waiting for trigger
																			  	 3= Triggered
																				 4= Uploading trace data*)
		TraceLength : 			UDINT; 											(*Length of trace data that has been uploaded*)
	END_VAR
	VAR_IN_OUT
		Dam :					DAM_REF; 										(*Reference to the DAM*)
	END_VAR
	VAR
		intern :					iDD_INTERN_TRACE_DAM; 						(*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_UpdateFirmware_DAM
	(*DD_UpdateFirmware: FB for DAM:
	This function block is used to update the appropriate firmware to a DAM. The update of the firmware is
	only allowed in Standby state. After successful file transfer, the DAM will automatically start
	updating and after that, a power-cycle is required to go back to normal operation.*)
	VAR_INPUT
		Execute : 			BOOL := FALSE; 						(*Starts the firmware update at a rising edge*)
		FileName : 			STRING[80] := ''; 						(*Filename of the firmware file on the PLC*)
		pDevice : UDINT; 		(*Pointer to the name of the PLC device on which the file is located*)
	END_VAR
	VAR_OUTPUT
		Done : 				BOOL; 								(*The firmware update process has successfully transferred the firmware to the device*)
		Busy : 				BOOL; 								(*The FB is not finished and new output values are to be expected*)
		Error : 				BOOL; 								(*Signals that an error has occurred within the FB*)
		ErrorInfo : 			DD_ERROR_DDS; 					(*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Dam : 				DAM_REF; 							(*Reference to the DAM*)
	END_VAR
	VAR
		intern : iDD_INTERN_UPDATE_FW_DDS; (*internal variable*)
		pDamIntern : REFERENCE TO iDD_DAM_REF_INTERN_DAM; (*internal variable of DAM*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_WriteParameter4_DAM
	(*DD_WriteParameter4: FB for DAM:
	FB to asynchronously write general objects of up to 4 bytes to the object dictionary of the DAM*)
	VAR_INPUT
		Execute : 		BOOL := FALSE; 			(*Write the value of the parameter at rising edge*)
		Index : 			UINT := 0; 				(*Index of the object to be written*)
		Subindex : 		USINT := 0; 				(*Subindex of the object to be written*)
		Length : 			USINT := 0; 				(*Length of the data in byte that shall be written*)
		Value : 			DWORD := 0; 				(*Value of the specified parameter*)
	END_VAR
	VAR_OUTPUT
		Done : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		AbortCode : 		DWORD; 				(*SDO abort code in case of error*)
	END_VAR
	VAR_IN_OUT
		Dam :			DAM_REF; 				(*Reference to the DAM*)
	END_VAR
	VAR
		intern : 			iDD_INTERN_SDO_WRITE_DAM; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK
