
FUNCTION_BLOCK AXIS_REF_DDS (*AXIS_REF: PLCopen for Danfoss Drive Servo with EtherCAT*)
	VAR_INPUT
		ModuleOk : BOOL; (*References ModuleOK from AutomationStudio*)
		NodeNumber : USINT; (* Node number of the device *)
		SlotName : STRING[80]; (*Name of the POWERLINK slot that this axis is physically connected to*)
		TPDO : ARRAY[0..8] OF UDINT; (* Data for IO Mapping *)
		DriveType : DD_DRIVE_TYPE_DDS := dd_ISD_DDS; (*Enumeration to specifiy the drive type, which will be used*)
	END_VAR
	VAR_OUTPUT
		RPDO : ARRAY[0..8] OF UDINT; (* Data for IO Mapping *)
	END_VAR
	VAR
		intern : iDD_AXIS_REF_INTERN_DDS; (* internal variable *)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_Power_DDS (*MC_Power: PLCopen FB for Danfoss Drive Servo:
	This function block controls the power stage (On or Off).*)
	VAR_INPUT
		Enable : BOOL := FALSE; (*As long as this input is true, power is being enabled*)
		TimeLimit : TIME := t#0ms; (*Time out after which an error is signalled, if the Status has not changed to true while Enable is true. 0 will disable the TimeLimit*)
	END_VAR
	VAR_OUTPUT
		Status : BOOL; (*Effective state of the power stage*)
		Valid : BOOL; (*If true, a valid set of outputs is available at the FB*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_POWER_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_Home_DDS (*MC_Home: PLCopen FB for Danfoss Drive Servo:
	This function block commands the axis to set its position to the value that is given as input.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Start the homing procedure at rising edge*)
		Position : DINT := 0; (*Absolute position when the reference signal is detected [u]*)
		Mode : DD_HOME_MODE_DDS := ddDirect_DDS; (*Defines the method used for homing. Depending on this selection, the appropriate input variables are used.*)
		Velocity : UDINT := 0; (*Value of speed during search for switch [u/s]*)
		VelocityOut : UDINT := 0; (*Value of speed during search for edge of switch [u/s]*)
		Acceleration : UDINT := 0; (*Value of the "Acceleration" [u/s]*)
		Deceleration : UDINT := 0; (*Value of the "Deceleration" [u/s]*)
		TorqueLimit : UINT := 16#FFFF; (*Maximum torque that is used for this motion*)
		BlockedVelocity : UDINT := 0; (*Axis assumes to be blocked, when the actual speed falls below the limit that is given here [u/s]*)
		BlockedDuration : UINT := 5; (*Axis assumes to be blocked, when the actual speed falls below the BlockedVelocity for the duration given here [ms]*)
		TimeLimit : TIME := t#0ms; (*Time out after which an error is signalled, if the homing procedure has not been completed. The homing procedure will be aborted automatically. 0= disabled*)
		DistanceLimit : UDINT := 0; (*Maximal distance in which the limit switch has to be reached. Otherwise the home procedure is aborted with error. 0 = disabled*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Reference known and set successfully*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		CommandAborted : BOOL; (*"Command" is aborted by another command*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_HOME_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_Stop_DDS (*MC_Stop: PLCopen FB for Danfoss Drive Servo:
	This function block commands a controlled motion stop and transfers the axis to
	the state "Stopping". It aborts any ongoing FB execution. While the axis is in 
	state "Stopping", no other FB can perform any motion on the same axis.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Start the action at rising edge*)
		Deceleration : UDINT := 0; (*Value of the "Deceleration" [u/s]. A value less or equal to 0 is not allowed. *)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Zero velocity reached*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		CommandAborted : BOOL; (*"Command" is aborted by switching off power (only possibility to abort)*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_PDO_EXC_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_Halt_DDS (*MC_Halt: PLCopen FB for Danfoss Drive Servo:
	This function block commands a controlled motion stop.
	This function block is used to stop the axis under normal operation conditions.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Start of action at rising edge*)
		Deceleration : UDINT := 0; (*Value of the "Deceleration" [u/s]. If value less or equal 0 is not allowed. *)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*"Zero velocity" reached *)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		CommandAborted : BOOL; (*"Command" is aborted by another command*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_PDO_EXC_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_MoveAbsolute_DDS (*MC_MoveAbsolute: PLCopen FB for Danfoss Drive Servo:
	The function block commands a controlled motion to a specified absolute position.
	This action completes with velocity zero if no further actions are pending.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Start the motion at rising edge*)
		Position : DINT := 0; (*Commanded "Position" for the motion (in technical unit [u]) (negative or positive)*)
		Velocity : DINT := 0; (*Value of the maximum velocity (not necessarily reached) in [u/s] (always positive)*)
		Acceleration : UDINT := 0; (*Value of the "Acceleration" (always positive) (increasing energy of the motor) [u/s]*)
		Deceleration : UDINT := 0; (*Value of the "Deceleration" (always positive) (decreasing energy of the motor) [u/s]*)
		Direction : MC_DIRECTION_DDS := mcShortestWay_DDS; (*Enum type (1-of-4 values: mcPositiveDirection_DDS, mcShortestWay_DDS, mcNegativeDirection_DDS, mcCurrentDirection_DDS)*)
		BufferMode : MC_BUFFER_MODE_DDS := mcAborting_DDS; (*Defines the chronological sequence of the FB*)
		TorqueLimit : UINT := 16#FFFF; (*Maximum torque that is used for this motion*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Commanded position finally reached*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Active : BOOL; (*Indicates that the FB has control on the axis*)
		CommandAborted : BOOL; (*"Command" is aborted by another command*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_MOTION_POS_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_MoveRelative_DDS (*MC_MoveRelative: PLCopen FB for Danfoss Drive Servo:
	This function block commands a controlled motion of a specified distance relative to the set position at the time of the execution.
	This action completes with velocity zero if no further actions are pending.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Start the motion at rising edge*)
		Distance : DINT := 0; (*Relative distance for the motion (in technical unit [u])*)
		Velocity : DINT := 0; (*Value of the maximum "Velocity" (not necessarily reached) (always positive)  [u/s]*)
		Acceleration : UDINT := 0; (*Value of the "Acceleration" (always positiive) (increasing energy of the motor) [u/s]*)
		Deceleration : UDINT := 0; (*Value of the "Deceleration" (always positive) (decreasing energy of the motor) [u/s]*)
		BufferMode : MC_BUFFER_MODE_DDS := mcAborting_DDS; (*Defines the chronological sequence of the FB*)
		TorqueLimit : UINT := 16#FFFF; (*Maximum torque that is used for this motion*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Commanded distance reached*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Active : BOOL; (*Indicates that the FB has control on the axis*)
		CommandAborted : BOOL; (*"Command" is aborted by another command*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_MOTION_POS_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_MoveAdditive_DDS (*MC_MoveAdditive: PLCopen FB for Danfoss Drive Servo:
	This function block commands a controlled motion of a specified relative distance additional to the most recent commanded position in the axis state "DiscreteMotion".
	This action completes with velocity zero if no further actions are pending.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Start the motion at rising edge*)
		Distance : DINT := 0; (*Relative distance for the motion (in technical unit [u])*)
		Velocity : DINT := 0; (*Value of the maximum "Velocity" (not necessarily reached) (always positive)  [u/s]*)
		Acceleration : UDINT := 0; (*Value of the "Acceleration" (always positiive) (increasing energy of the motor) [u/s]*)
		Deceleration : UDINT := 0; (*Value of the "Deceleration" (always positive) (decreasing energy of the motor) [u/s]*)
		BufferMode : MC_BUFFER_MODE_DDS := mcAborting_DDS; (*Defines the chronological sequence of the FB*)
		TorqueLimit : UINT := 16#FFFF; (*Maximum torque that is used for this motion*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Commanded position finally reached*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Active : BOOL; (*Indicates that the FB has control on the axis*)
		CommandAborted : BOOL; (*"Command" is aborted by another command*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_MOTION_POS_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_MoveVelocity_DDS (*MC_MoveVelocity: PLCopen FB for Danfoss Drive Servo:
	The function block commands a never ending controlled motion at a specified velocity.
	To stop the motion, the FB has to be interrupted by another FB issuing a new command.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Start the motion at rising edge*)
		Velocity : DINT := 0; (*Value of the maximum velocity [u/s]. Can be a signed value.*)
		Acceleration : UDINT := 0; (*Value of the "Acceleration" (always positiive) (increasing energy of the motor) [u/s]*)
		Deceleration : UDINT := 0; (*Value of the "Deceleration" (always positive) (decreasing energy of the motor) [u/s]*)
		TorqueLimit : UINT := 16#FFFF; (*Maximum torque that is used for this motion*)
	END_VAR
	VAR_OUTPUT
		InVelocity : BOOL; (*Commanded velocity reached*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		CommandAborted : BOOL; (*"Command" is aborted by another command*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_PDO_EXC_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_TorqueControl_DDS (*MC_TorqueControl: PLCopen FB for Danfoss Drive Servo:
	This function block continuously exerts a torque or force of the specified magnitude. This magnitude is
	approached using a defined ramp ("TorqueRamp").*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Start the motion at rising edge*)
		Torque : INT := 0; (*Value of the torque. Given in per thousands of rated torque.*)
		TorqueRamp : UDINT := 0; (*The maximum time derivative of the set value of the torque or force (in technical unit per sec [u/s])*)
		Velocity : DINT := 0; (*Absolute value of the maximum velocity [u/s]*)
	END_VAR
	VAR_OUTPUT
		InTorque : BOOL; (*Setpoint value of torque or force equals the commanded value*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		CommandAborted : BOOL; (*"Command" is aborted by another command*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_PDO_EXC_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_ReadParameter_DDS (*MC_ReadParameter: PLCopen FB for Danfoss Drive Servo:
	This function block returns the value of a specific parameter. *)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the value of the parameter continuously while enabled*)
		ParameterNumber : INT := 0; (*Number of the parameter*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		AbortCode : DWORD; (*Abort code that was sent by the drive in case of aborted SDO transfer*)
		Value : DWORD; (*Value of the specified parameter*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_PARAM_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_ReadBoolParameter_DDS (*MC_ReadParameter: PLCopen FB for Danfoss Drive Servo:
	This function block returns the value of a specific parameter. *)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the value of the parameter continuously while enabled*)
		ParameterNumber : INT := 0; (*Number of the parameter*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		AbortCode : DWORD; (*Abort code that was sent by the drive in case of aborted SDO transfer*)
		Value : BOOL; (*Value of the specified parameter*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_PARAM_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_WriteParameter_DDS (*MC_WriteParameter: PLCopen FB for Danfoss Drive Servo:
	This function block modifies the value of a specific parameter.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Write the value of the parameter at rising edge*)
		ParameterNumber : INT := 0; (*Number of the parameter*)
		Value : DWORD := 0; (*New value of the specified parameter*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		AbortCode : DWORD; (*Abort code that was sent by the drive in case of aborted SDO transfer*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_WRITE_PARA_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_ReadActualPosition_DDS (*MC_ReadActualPosition: PLCopen FB for Danfoss Drive Servo:
	This function block returns the actual position as long as "Enable" is set.*)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the value of the parameter continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		Position : DINT; (*New absolute position (in axis' unit [u])*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_ReadActualVelocity_DDS (*MC_ReadActualVelocity: PLCopen FB for Danfoss Drive Servo:
	This function block returns the actual velocity as long as "Enable" is set. *)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the value of the parameter continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		Velocity : DINT; (*The value of the actual velocity (in axis' unit [u/s])*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_ReadActualTorque_DDS (*MC_ReadActualTorque: PLCopen FB for Danfoss Drive Servo:
	This function block returns the actual torque as long as "Enable" is set.*)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the value of the parameter continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		Torque : INT; (*The value of the actual torque or force (in per thousands of rated torque)*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_ReadStatus_DDS (*MC_ReadStatus: PLCopen FB for Danfoss Drive Servo:
	This function block returns in detail the status of the state diagram of the selected axis.*)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the values of the parameters continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid set of outputs is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		ErrorStop : BOOL; (*Axis is in PLCopen state Errorstop*)
		Disabled : BOOL; (*Axis is in PLCopen state  Disabled*)
		Stopping : BOOL; (*Axis is in PLCopen state  Stopping*)
		Homing : BOOL; (*Axis is in PLCopen state Homing*)
		Standstill : BOOL; (*Axis is in PLCopen state Standstill*)
		DiscreteMotion : BOOL; (*Axis is in PLCopen state Discrete Motion*)
		ContinuousMotion : BOOL; (*Axis is in PLCopen state Continuous Motion*)
		SynchronizedMotion : BOOL; (*Axis is in PLCopen state Synchronized Motion*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_ReadMotionState_DDS (*MC_ReadMotionState: PLCopen FB for Danfoss Drive Servo:
	This function block returns in detail the status of the axis with respect to the motion currently in progress.*)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the values of the parameters continously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid set of outputs is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		ConstantVelocity : BOOL; (*Velocity is constant. Velocity may be 0. For the actual value a window is applicable (window is vendor specific)*)
		Standstill : BOOL; (*Velocity is constant. Velocity is 0.*)
		Accelerating : BOOL; (*Increasing the absolute value of the velocity*)
		Decelerating : BOOL; (*Decreasing the absolute value of the velocity*)
		DirectionPositive : BOOL; (*Signals that the position is increasing*)
		DirectionNegative : BOOL; (*Signals that the position is decreasing*)
		LimitActive : BOOL; (*Signals that an internal limit is active*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_READ_MOTION_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_ReadAxisInfo_DDS (*MC_ReadAxisInfo: PLCopen FB for Danfoss Drive Servo:
	This function block reads information concerning an axis, like modes, inputs directly related to the axis and certain status information.*)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the axis information constantly while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid set of outputs is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		HomeAbsSwitch : BOOL; (*Digital home switch input is active*)
		LimitSwitchPos : BOOL; (*Positive hardware end switch is active*)
		LimitSwitchNeg : BOOL; (*Negative hardware end switch is active*)
		SWLimitPos : BOOL; (*Positive software limit is active*)
		SWLimitNeg : BOOL; (*Negative software limit is active*)
		CommunicationReady : BOOL; (*"Network" is initialized and ready for communication*)
		ReadyForPowerOn : BOOL; (*Drive is ready to be enabled (power on)*)
		PowerOn : BOOL; (*If TRUE shows that the power stage is switched ON*)
		SafeTorqueOff : BOOL; (*If TRUE: STO is activated = Safety voltage is missing*)
		IsHomed : BOOL; (*The absolute reference position is known for the axis (axis is homed)*)
		AxisWarning : BOOL; (*Warning(s) on the axis is present*)
		AxisError : BOOL; (*Error(s) on the axis is present*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_ReadAxisError_DDS (*MC_ReadAxisError: PLCopen FB for Danfoss Drive Servo:
	This function block presents general axis errors not relating to the function blocks
	(for instance axis errors, drive errors, communication errors).*)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the value of the axis error constantly while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		AxisErrorID : WORD; (*The value of the axis error*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_ReadDigitalInput_DDS (*MC_ReadDigitalInput: PLCopen FB for Danfoss Drive Servo:
	This function block returns the actual value of the digital input, referenced by the InputNumber as long as "Enable" is set. *)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the value of the selected input signal continuously while enabled*)
		InputNumber : INT := 1; (*Selects the input. Value range {1; 2}*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		Value : BOOL; (*The value of the selected input signal*)
	END_VAR
	VAR_IN_OUT
		Input : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_ReadDigitalOutput_DDS (*MC_ReadDigitalOutput: PLCopen FB for Danfoss Drive Servo:
	This function block returns the actual value of the digital output as long as "Enable" is set. *)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the value of the output signal continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		Value : BOOL; (*The value of the output signal*)
	END_VAR
	VAR_IN_OUT
		Output : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_Reset_DDS (*MC_Reset: PLCopen FB for Danfoss Drive Servo:
	This function block makes the transition from the state "ErrorStop" to "Disabled" by resetting all internal axis-related errors - 
	it does not affect the output of the FB instances.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Resets all internal axis-related errors*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Disabled state is reached*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_RESET_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_CamTableSelect_DDS (*MC_CamTableSelect: PLCopen FB for Danfoss Drive Servo:
	This function block transfers the CAM profile and the configuration to the drive.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Start transfer of CAM table at rising edge*)
		CamTableID : USINT := 0; (*Identifier of CAM table to be used, needs to be linked to input of MC_CamIn_DDS*)
		MasterAbsolute : BOOL := FALSE; (*TRUE = absolute; FALSE = relative coordinates*)
		SlaveAbsolute : BOOL := FALSE; (*TRUE = absolute; FALSE = relative coordinates*)
		Cyclic : BOOL := TRUE; (*TRUE = cyclic; FALSE = non-cyclic*)
		pDevice : UDINT; (*Pointer to the name of the PLC device on which the file is located*)
		PatternFile : STRING[80] := ''; (*Name of the pattern file on the PLC in case the CAM profile contains a pattern alignment*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*CAM profile and configuration has been downloaded; Parsing was successful*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		ParseError : WORD; (*Detailed information on the type of error in case of CAM parsing failure*)
		ParseErrorDebug : DWORD; (*Depending on the cause given in the ParseError output, additional debug information is given here.
																	 See documentation for interpretation of this data.*)
	END_VAR
	VAR_IN_OUT
		Slave : AXIS_REF_DDS; (*Reference to the slave axis*)
		CamTable : STRING[80]; (*Reference to CAM description (filename of CAM profile on the PLC)*)
	END_VAR
	VAR
		intern : iDD_INTERN_CAM_TABLE_SEL_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_CamIn_DDS (*MC_CamIn: PLCopen FB for Danfoss Drive Servo:
	This function block engages the CAM.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Start the CAM at rising edge*)
		MasterOffset : DINT := 0; (*Offset of the master shaft to cam*)
		SlaveOffset : REAL := 0.0; (*Offset of slave shaft. Given in revolutions*)
		CamTableID : USINT := 0; (*Identifier of CAM table to be used, linked to output of MC_CamTableSelect_DDS*)
		ChangeImmediate : BOOL := FALSE; (*TRUE: Abort the currently running CAM immediately
																						 FALSE: let the currently running CAM finish first*)
		UseBlendingDistance : BOOL := FALSE; (*FALSE: Automatically blend to the beginning of the new CAM
																						 TRUE: Use the BlendingDistance at minimum for blending*)
		BlendingDistance : UDINT := 0; (*Used in the direction of the Master, minimum length used for blending to the new CAM if UseBlendingDistance = TRUE*)
		SwitchControlSetByCam : BOOL := TRUE; (*TRUE: Control parameter set selection is handled by the CAM itself
																						 FALSE: Control parameter set selection is hone using DD_SelectControlParamSet_DDS*)
		StartDelay : DD_CAM_DELAY_MODE_DDS := ddNoDelay_DDS; (*Influences the activation behaviour of a CAM. Only valid for Master relative CAMs.*)
	END_VAR
	VAR_OUTPUT
		InSync : BOOL; (*Is TRUE if the set value equals the commanded value.
																						 The drive is currently running on the specified CAM table*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Active : BOOL; (*The selected CAM table is activated in the axis.*)
		CommandAborted : BOOL; (*"Command" is aborted by another command*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		EndOfProfile : BOOL; (*Pulsed output signalling the cylic end of the CAM profile. It is displayed every time the end of the cam profile is reached.
																						 In reverse direction, the EndOfProfile is displayed also at the end of the cam profile (in this case the first point of the cam profile).*)
	END_VAR
	VAR_IN_OUT
		Master : GUIDE_VALUE_REF_DDS; (*Reference to the master axis*)
		Slave : AXIS_REF_DDS; (*Reference to the slave axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_MOTION_CAM_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_CamScaling_DDS (*DD_CamScaling: FB for Danfoss Drive Servo:
	This function block sets new scaling values for the CAM. 
	Note that these values are written using SDO communication. So the timing can not be guaranteed.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Starts the parameter transfer to the drive at a rising edge*)
		MasterScalingNumerator : DINT := 1; (*Numerator for the CAM master scaling. From the slave point of view the master overall profile is multplied with this factor.*)
		MasterScalingDenominator : DINT := 1; (*Denominator for the CAM master scaling. From the slave point of view the master overall profile is devided by this factor.*)
		SlaveScalingNumerator : DINT := 1; (*Numerator for the CAM slave scaling. The overall slave profile is multplied with this factor.*)
		SlaveScalingDenominator : DINT := 1; (*Denominator for the CAM slave scaling. The overall slave profile is devided by this factor.*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The parameters have been set*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Slave : AXIS_REF_DDS; (*Reference to the slave axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_WRITE_SCALING_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_DigitalCamSwitch_DDS (*DD_DigitalCamSwitch: FB for Danfoss Drive Servo:
	This function block is the analog to switches on a motor shaft.*)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Control the digital CAM switching functionality*)
		EnableSwitches : BOOL := FALSE; (*Enables / disables the digital CAM switching functionality*)
	END_VAR
	VAR_OUTPUT
		InOperation : BOOL; (*The digital CAM switching functionality is enabled*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Output : AXIS_REF_DDS; (*Reference to the axis / signal output*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_GetInertia_DDS (*DD_GetInertia: FB for Danfoss Drive Servo:
	This function block is used to determine the inertia of the drive system. *)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Starts the inertia measurement at rising edge*)
		VelocityLimit : UDINT := 0; (*Maximum velocity that is used during the measurement [u/s]*)
		TorqueLimit : UINT := 16#FFFF; (*Maximum torque that is used for this motion. This value has to be given in per thousand of rated torque*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The inertia measurement has successfully been finished and the value is available*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		CommandAborted : BOOL; (*"Command" is aborted by another command*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		Inertia : REAL; (*Measured inertia; Given in kg m*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_INERTIA_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_PrepareDigCamSwitch_DDS (*DD_PrepareDigCamSwitch: FB for Danfoss Drive Servo:
	This function block prepares the drive with the digital CAM switches by handing over the digital CAM switches information.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Start transfer of digital CAM switches data at rising edge*)
		FileName : STRING[80] := ''; (*Reference to digital CAM switches description (filename of the digital CAM switches data on the PLC)*)
		pDevice : UDINT; (*Pointer to the name of the PLC device on which the file is located*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Digital CAM switches data has been transferred and parsed successfully*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		ParseError : WORD; (*Detailed information on the type of error in case of a parsing failure*)
		ParseErrorDebug : DWORD; (*Depending on the cause given in the ParseError output, additional debug information is given here.
															 See documentation for interpretation of this data.*)
	END_VAR
	VAR_IN_OUT
		Output : AXIS_REF_DDS; (*Reference to the slave axis / signal output*)
	END_VAR
	VAR
		intern : iDD_INTERN_PREP_DCS_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ProduceGuideValue_DDS (*DD_ProduceGuideValue: FB for Danfoss Drive Servo:
	This function block is used to calculate a virtual guide value within the PLC. Only linear ramps are supported.*)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Updates the guide value reference as long as Enable = TRUE.
															 If Enable = FALSE, the guide value stops immediately!*)
		GuideVelocity : REAL := 0.0; (*Velocity of the guide value. Has to be given in rps.*)
		GuideAcceleration : REAL := 0.0; (*Acceleration value used while increasing the velocity of the guide value.
															 Has to be given in rps/s. Only positive values are allowed.,*)
		GuideDeceleration : REAL := 0.0; (*Deceleration value used while decreasing the velocity of the guide value.
															 Has to be given in rps/s. Only positive values are allowed.*)
	END_VAR
	VAR_OUTPUT
		InGuideVelocity : BOOL; (*Commanded guide velocity reached*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		GuideValueProducer : GUIDE_VALUE_REF_DDS; (*Reference to the guide value reference*)
	END_VAR
	VAR
		intern : iDD_INTERN_GUIDEVALUE_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadAnalogInput_DDS (*DD_ReadAnalogInput: FB for Danfoss Drive Servo:
	This function block returns the actual value of the analogue input, referenced by the InputNumber as long as "Enable" is set.*)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the value of the selected input signal continuously while enabled*)
		InputNumber : INT := 1; (*Selects the input. Value range {1; 2}*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		Value : REAL; (*The value of the selected input signal*)
	END_VAR
	VAR_IN_OUT
		Input : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadAxisWarning_DDS (*DD_ReadAxisWarning: FB for Danfoss Drive Servo:
	This function block presents general axis warnings not relating to the function blocks (for instance STO).*)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the value of the axis warning constantly while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		AxisWarningID : WORD; (*The value of the axis warning*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadParameter4_DDS (*DD_ReadParameter4: FB for Danfoss Drive Servo:
	FB to asynchronously read general objects of up to 4 bytes from the object dictionary of the drive*)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the value of the object continuously while enabled*)
		Index : UINT := 0; (*Index of the object to be read*)
		Subindex : USINT := 0; (*Subindex of the object to be read*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		AbortCode : DWORD; (*Abort code that was sent by the drive in case of aborted SDO transfer*)
		Value : DWORD; (*Value of the specified object*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadParameter_DDS (*DD_ReadParameter: FB for Danfoss Drive Servo:
	FB to asynchronously read general objects of more than 4 bytes from the object dictionary of the drive*)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the value of the object continuously while enabled*)
		Index : UINT := 0; (*Index of the object to be read*)
		Subindex : USINT := 0; (*Subindex of the object to be read*)
		pBuffer : UDINT := 0; (*Pointer to a buffer where the read data shall be placed; use ADR() function*)
		BufferSize : UDINT := 0; (*Maximum size of the read data (size of the provided buffer given in Byte; use SIZEOF() function)*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		AbortCode : DWORD; (*Abort code that was sent by the drive in case of aborted SDO transfer*)
		DataLength : UDINT; (*Length of the read data (in byte)*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadVersion_DDS (*DD_ReadVersion: FB for Danfoss Drive Servo:
	FB to read the version information out of the drive*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Read the information at rising edge*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*A valid set of outputs is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		SerialNumber : STRING[18]; (*Serial number of the drive*)
		MajorVersionNo : UINT; (*Major firmware number*)
		MinorVersionNo : UINT; (*Minor firmware number*)
		PatchVersionNo : UINT; (*Patch firmware number*)
		PackageLabel : STRING[20]; (*Label of the software package*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_READ_VERS_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_SelectControlParamSet_DDS (*DD_SelectControlParamSet: FB for Danfoss Drive Servo:
	This function block does the selection of the used control parameter set of the drive.
	As long as Enable is TRUE, the input Set is evaluated. if input Enable is FALSE, the last value will be used.*)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Selects the control parameter set*)
		Set : USINT := 1; (*Number of the control parameter set that should be used. Valid values: 1 or 2*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_Trace_DDS (*DD_Trace: FB for Danfoss Drive Servo:
	This function block is used to start a trace in the axis with the settings given in the input variables. After
	sending the settings and starting the trace, the FB will poll the axis until the data has been recorded
	and afterwards uploads the data automatically.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Starts the trace functionality at rising edge and keeps on polling*)
		Abort : BOOL := FALSE; (*Abort the on-going trace. Note that new values are only evaluated on a rising edge of Execute.*)
		pTraceBuffer : UDINT := 0; (*Reference to a buffer where the read trace data shall be placed; use ADR() function*)
		TraceBufferSize : UDINT := 0; (*Size of the trace buffer (size of the provided buffer given in Byte; use SIZEOF() function)*)
		SamplingRate : DD_SAMPLING_RATE_DDS := ddFastTask_DDS; (*Sampling rate of the trace*)
		SampleCount : UINT := 4000; (*Number of samples to be traced per channel*)
		SubSampling : UINT := 1; (*Multiplier to adjust time difference between trace samples*)
		SignalIDs : ARRAY[0..7] OF UDINT := [8(0)]; (*Holds the IDs of the signals that should be traced*)
		TriggerID : UDINT := 0; (*ID of the signal used for triggering; Use 0 to have instantaneous tracing.*)
		TriggerPoint : USINT := 10; (*Holds the information on how many pre-trigger history is traced [in percentage]*)
		TriggerLevel : REAL := 0.0; (*Level upon which will be triggered*)
		TriggerSlope : BOOL := TRUE; (*TRUE: will trigger on rising slope; FALSE: will trigger on falling slope*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The trace has successfully been recorded and read from the device.*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		CommandAborted : BOOL; (*Trace has been aborted.*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		Status : USINT; (*Holds the status of the tracing:
																				 1= Trace configured and started
																				 2= Waiting for trigger
																				 3= Triggered
																				 4= Uploading trace data*)
		TraceLength : UDINT; (*Length of trace data that has been uploaded (given in Bytes; note that one sample (REAL) is 4 Bytes long)*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_TRACE_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_UpdateFirmware_DDS (*DD_UpdateFirmware: FB for Danfoss Drive Servo:
	This function block is used to update the appropriate firmware to a drive. The update of the firmware is
	only allowed in an unpowered drive state. After successful file transfer, the drive will automatically start
	updating and after that, a power-cycle is required to go back to normal operation.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Starts the firmware update at a rising edge*)
		FileName : STRING[80] := ''; (*Filename of the firmware file on the PLC*)
		pDevice : UDINT; (*Pointer to the name of the PLC device on which the file is located*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The firmware update process has successfully transferred the firmware to the device*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_UPDATE_FW_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_WriteDigitalOutput_DDS (*DD_WriteDigitalOutput: FB for Danfoss Drive Servo:
	This function block writes a value to the digital output of the drive. As long as "Enable" is set, the input Value is evaluated. 
	If the input "Enable" is FALSE; the last value will be used.*)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Write the value of the digital output*)
		Value : BOOL := FALSE; (*The value of the output (TRUE: set; FALSE: clear)*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Output : AXIS_REF_DDS; (*Reference to the axis / signal output*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_WriteParameter4_DDS (*DD_WriteParameter4: FB for Danfoss Drive Servo:
	FB to asynchronously write general objects of up to 4 bytes to the object dictionary of the drive*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Write the value of the parameter at rising edge*)
		Index : UINT := 0; (*Index of the object to be written*)
		Subindex : USINT := 0; (*Subindex of the object to be written*)
		Length : USINT := 0; (*Length of the data in byte that shall be written*)
		Value : DWORD := 0; (*Value of the specified parameter*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		AbortCode : DWORD; (*SDO abort code in case of error*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_WRITE_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_WriteParameter_DDS (*DD_WriteParameter: FB for Danfoss Drive Servo:
	FB to asynchronously write general objects of more than 4 bytes to the object dictionary of the drive*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Write the value of the object at rising edge*)
		Index : UINT := 0; (*Index of the object to be written*)
		Subindex : USINT := 0; (*Subindex of the object to be written*)
		Length : USINT := 0; (*Length of the data in byte that shall be written*)
		pBuffer : UDINT := 0; (*Pointer to the buffer that contains the data that shall be written*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		AbortCode : DWORD; (*Abort code that was sent by the drive in case of aborted SDO transfer*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_WRITE_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_TouchProbe_DDS (*MC_TouchProbe: PLCopen FB for Danfoss Drive Servo:
	This function block is used to record an axis position at a trigger event.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Start touch probe recording at rising edge*)
		WindowOnly : BOOL := FALSE; (*If SET, only use the window (defined hereunder) to accept trigger events*)
		FirstPosition : DINT := 0; (*Start position from where (positive direction) trigger events are accepted (in technical units[u]). Value included in window.*)
		LastPosition : DINT := 0; (*Stop position of the window (in technical units [u]). Value included in window.*)
		EdgeMode : DD_EDGE_MODE_DDS := ddPositiveEdge_DDS; (*Indicates on which input events the drive triggers.*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Trigger event recorded*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		CommandAborted : BOOL; (*"Command" is aborted by another command (MC_AbortTrigger_DDS)*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		RecordedPosition : DINT; (*Position where trigger event occurred (in technical units [u]).*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
		TriggerInput : UINT; (*Reference to the trigger signal source. 1 for the first digital input; 2 for the second digital input.*)
	END_VAR
	VAR
		intern : iDD_INTERN_TOUCH_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_AbortTrigger_DDS (*MC_AbortTrigger: PLCopen FB for Danfoss Drive Servo:
	This function block is used to abort function blocks which are connected to trigger events (see MC_TouchProbe_DDS).*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Abort trigger event*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Trigger functionality aborted*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
		TriggerInput : UINT; (*Reference to the trigger signal source. 1 for the first digital input; 2 for the second digital input.*)
	END_VAR
	VAR
		intern : iDD_INTERN_TOUCH_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_SetFollowSegment_DDS (* For Advanced CAM only: Sets the following segment of a node *)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Sends the switching command*)
		NodeID : UINT := 0; (*ID of the node where the following segment should be changed*)
		SegmentID : UINT := 0; (*ID of the segment that should be used from now on after the node with NodeID.*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The command has been sent*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_CAM_CMD_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_SetSegmentParameter_DDS (* For Advanced CAM only: Sends a parameter value to the segment with the passed SegmentID *)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Sends the parameter to the axis*)
		SegmentID : UINT := 0; (*ID of the segment that should use this parameter*)
		Angle : REAL := 0.0; (*Angle parameter that should be used by the given segment. Angle has to be given in (slave) revolutions*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The parameter has been sent*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_CAM_CMD_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_NodeNotification_DDS (* For Basic and Advanced CAMs: Enables / disables the sending of a messing when passing a node *)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Sends the command to enable / disable node notification*)
		NodeID : UINT := 0; (*ID of the node that should (not) send a notification when it is passed*)
		SendNote : BOOL := FALSE; (*FALSE: No notification will be sent; TRUE: Notifiaction is sent + information of following segment*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The parameter has been sent*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_CAM_CMD_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_GearIn_DDS (*MC_GearIn: PLCopen FB for Danfoss Drive Servo:
	Commands a synchronized motion with the master*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Starts the gearing process at the rising edge*)
		RatioNumerator : INT := 1; (*Gear ratio numerator*)
		RatioDenominator : INT := 1; (*Gear ratio denominator*)
		Acceleration : UDINT := 0; (*Acceleration for gearing in procedure*)
		Deceleration : UDINT := 0; (*Deceleration for gearing in procedure*)
	END_VAR
	VAR_OUTPUT
		InGear : BOOL; (*Is TRUE if the set value equals the commanded value*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		CommandAborted : BOOL; (*"Command" is aborted by another command*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Master : GUIDE_VALUE_REF_DDS; (*Reference to the master axis*)
		Slave : AXIS_REF_DDS; (*Reference to the slave axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_PDO_EXC_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK MC_GearInPos_DDS (*MC_GearInPos: PLCopen FB for Danfoss Drive Servo:
	Commands a synchronized motion with the master from the synchronization point onwards*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Starts the gearing process at the rising edge*)
		RatioNumerator : INT := 1; (*Gear ratio numerator*)
		RatioDenominator : INT := 1; (*Gear ratio denominator*)
		MasterSyncPosition : UDINT := 0; (*The position of the master where the slave is in sync with the master*)
		SlaveSyncPosition : DINT := 0; (*Slave position at which the axes are running in sync*)
		SyncMode : MC_SYNC_MODE_DDS := mcShortest_DDS; (*Defines the way to synchronize (like mcShortest_DDS, mcCatchUp_DDS, mcSlowDown_DDS)*)
		MasterSyncDirection : BOOL := FALSE; (*FALSE: Master start distance is in the positve direction of the guide value
																				  TRUE: Master start distance is in the negative direction of the guide value*)
		MasterStartDistance : UDINT := 0; (*Master distance for gear in procedure (when the slave axis is started to get into synchronization)*)
		Velocity : DINT := 0; (*Maximum velocity during the time difference StartSync and InSync*)
		Acceleration : UDINT := 0; (*Acceleration for gearing in procedure*)
		Deceleration : UDINT := 0; (*Deceleration for gearing in procedure*)
	END_VAR
	VAR_OUTPUT
		StartSync : BOOL; (*Commanded gearing starts*)
		InSync : BOOL; (*Is TRUE if the set value equals the commanded value (is calculated set of values derived of master position and gear ratio)*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		CommandAborted : BOOL; (*"Command" is aborted by another command*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Master : GUIDE_VALUE_REF_DDS; (*Reference to the master axis*)
		Slave : AXIS_REF_DDS; (*Reference to the slave axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_PDO_EXC_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_BrakeHandling_DDS (*DD_BrakeHandling: FB for Danfoss Drive Servo:
	This function block overwrites the status of the mechanical brake.
	Note that is is only a holding brake and it may not be sued within normal operation.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Overwrites the brake state at rising edge*)
		Lift : BOOL := FALSE; (*TRUE will overwrite the brake automatic and will lift it; FALSE for releasing the brake*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The brake has successfully been modified.*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected.*)
		Error : BOOL; (*Signals that an error has occurred within the FB.*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_WRITE_PARA_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_GoToSetpoint_DDS (* For Basic and Advanced CAMs: Commands a movement to the set point of the CAM while the Guide Value velocity is 0.
	This is used e.g. when starting up a CAM and the axis is at another position.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Sends the command to start the movement*)
		Direction : MC_DIRECTION_DDS := mcShortestWay_DDS; (*Direction to be used to go to the set point*)
		Duration : UINT := 0; (*Duration of this movement. Given in ms.*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The command has been sent*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_CAM_CMD_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadCamInfo_DDS (*DD_ReadCamInfo: FB for Danfoss Drive Servo: For Basic and Advanced CAMs.
	This function block returns CAM related status information as long as "Enable" is set. *)
	VAR_INPUT
		Enable : BOOL := FALSE; (*Get the information continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid output is available at the FB*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
		StatusCode : UINT; (*The status code differentiates what kind of information it is*)
		Parameter1 : UINT; (*Parameter 1: Meaning depends on the StatusCode*)
		Parameter2 : UINT; (*Parameter 2: Meaning depends on the StatusCode*)
		Parameter3 : UINT; (*Parameter 3: Meaning depends on the StatusCode*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_TRIGGER_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_RotationStop_DDS (* For Basic and Advanced CAMs: Stops the processing of the CAM for one master cycle. *)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Stops the processing of the CAM for this master cycle.*)
		StopMode : DD_STOPPING_MODE_DDS := ddCoasting_DDS; (*Specifies the way of stopping the axis for this master cycle.*)
		Deceleration : REAL := 0; (*Deceleration value that is used for the ramping procedure. Only positive values are allowed. Given in rps/s.*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The command has been sent.*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_CAM_CMD_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_WriteConfigParameter_DDS (*DD_WriteConfigParameter: FB for Danfoss Drive Servo:
	This function block is used to transfer config parameter file to a drive. 
	After successful file transfer, a power-cycle is required to go back to normal operation.*)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Start transfer of config parameter file at rising edge*)
		FileName : STRING[80] := ''; (*Filename of the config parameter file on the PLC*)
		pDevice : UDINT; (*Pointer to the name of the PLC device on which the file is located*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The firmware update process has successfully transferred the firmware to the device*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_WRITE_CONFIG_PARA_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ResAmplitudeCalibration_DDS (*DD_ResAmplitudeCalibration_DDS: FB for Danfoss Drive Servo:
	This function block is used to determine the feedback sensor offset. Only activate the function block when the axis is in state Standstill.
	This function block can be used only for DSD, SDM 511 and SDM 512 drives.
	Only values >0 are allowed for the inputs TargetVelocity, Acceleration and Deceleration. *)
	VAR_INPUT
		Execute : BOOL := FALSE; (*Starts the resolver amplitude calibration at rising edge.*)
		TargetVelocity : DINT := 0; (*Value of the target velocity [u/s]. Can be a signed value.*)
		Acceleration : UDINT := 0; (*Value of the "Acceleration" (always positiive) [u/s]*)
		Deceleration : UDINT := 0; (*Value of the "Deceleration" (always positive) [u/s]*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Resolver calibration completed successfully and the value is available.*)
		Busy : BOOL; (*The function block is not finished and new output values are to be expected.*)
		CommandAborted : BOOL; (*"Command" is aborted by another command*)
		Error : BOOL; (*An error has occurred within the function block.*)
		ErrorInfo : DD_ERROR_DDS; (*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to the axis*)
	END_VAR
	VAR
		intern : iDD_INTERN_RES_AMPL_CALIBR_DDS; (*internal variable*)
		pAxisIntern : REFERENCE TO iDD_AXIS_REF_INTERN_DDS; (*internal variable of axis*)
	END_VAR
END_FUNCTION_BLOCK
