
FUNCTION_BLOCK DD_Power_PSM
	(*DD_Power: PLCopen FB for PSM:
	This function block controls the DC link voltage on the output lines (not the FB itself)*)
	VAR_INPUT
		Enable : 			BOOL := FALSE;				(*As long as this input is true, DC link voltage is being enabled on the output lines*)
		TimeLimit : 		TIME := t#0ms;				(*Time out after which an error is signalled, if the Status has not changed to true while Enable is true. 0 will disable the TimeLimit*)
	END_VAR
	VAR_OUTPUT
		Status : 			BOOL; 						(*Effective state of the DC link voltage on the output lines*)
		Valid : 			BOOL; 						(*If true, a valid set of outputs is available at the FB*)
		Error : 			BOOL; 						(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 			(*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Psm : 			PSM_REF;					(*Reference to the PSM*)
	END_VAR
	VAR
		intern : iDD_INTERN_POWER_DDS; (*internal variable*)
		pPsmIntern : REFERENCE TO iDD_PSM_REF_INTERN_PSM; (*internal variable of SAB*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadDcLinkPower_PSM
	(*DD_ReadDcLinkPower: FB for PSM:
	This function block returns the value of the actual DC link power as long as "Enable" is set.*)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 				(*Get the value of the parameter continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid :			BOOL; 						(*A valid output is available at the FB*)
		Busy : 			BOOL; 						(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 						(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 			(*Error identification and instance identifier*)
		Power : 			REAL; 						(*The value of the actual DC link power [Ampere]*)
	END_VAR
	VAR_IN_OUT
		Psm : 			PSM_REF; 					(*Reference to the PSM*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadDcLinkVoltage_PSM
	(*DD_ReadDcLinkVoltage: FB for PSM:
	This function block returns the value of the actual DC link voltage as long as "Enable" is set.*)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 			(*Get the value of the parameter continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL;					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		Voltage : 		INT; 					(*The value of the actual DC link voltage [V]*)
	END_VAR
	VAR_IN_OUT
		Psm : 			PSM_REF; 				(*Reference to the PSM*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadDigitalInput_PSM
	(*MC_ReadDigitalInput: PLCopen FB for PSM:
	This function block returns the actual value of the digital input, referenced by the InputNumber as long as "Enable" is set. *)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 			(*Get the value of the selected input signal continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS;		(*Error identification and instance identifier*)
		Value : 			BOOL; 					(*The value of the selected input signal*)
	END_VAR
	VAR_IN_OUT
		Input : 			PSM_REF; 				(* Reference to the PSM *)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadDigitalOutput_PSM
	(*DD_ReadDigitalOutput: FB for PSM:
	This function block reads a value to the digital output of the PSM. As long as "Enable" is set, the input Value is evaluated. 
	If the input "Enable" is FALSE; the last value will be used.*)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 		(*Read  the value of the digital output*)
		OutputNumber : 	INT := 1; 				(*Selects the output. Value range {1; 2}*)
	END_VAR
	VAR_OUTPUT
		Valid : 			BOOL; 				(*A valid output is available at the FB*)
		Busy : 			BOOL; 				(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 				(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 	(*Error identification and instance identifier*)
		Value : 			BOOL := FALSE; 		(*The value of the output (TRUE: set; FALSE: clear)*)
	END_VAR
	VAR_IN_OUT
		Output : 			PSM_REF; 			(*Reference to the axis / signal output*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
		pPsmIntern : REFERENCE TO iDD_PSM_REF_INTERN_PSM; (*internal variable of PSM *)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadParameter4_PSM
	(*DD_ReadParameter4: FB for PSM:
	FB to asynchronously read general objects of up to 4 bytes from the object dictionary of the PSM *)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 			(*Get the value of the parameter continuously while enabled*)
		Index : 			UINT := 0; 				(*Index of the object to be read*)
		Subindex : 		USINT := 0; 				(*Subindex of the object to be read*)
	END_VAR
	VAR_OUTPUT
		Valid : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		AbortCode : 		DWORD; 				(*SDO abort code in case of error*)
		Value : 			DWORD; 				(*Value of the specified parameter*)
	END_VAR
	VAR_IN_OUT
		Psm : 			PSM_REF; 				(*Reference to the PSM*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_PSM; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadPsmError_PSM
	(*DD_ReadPsmError: FB for PSM:
	This function block presents general PSM errors not relating to the function blocks
	(for instance PSM errors, voltage errors, communication errors).*)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 			(*Get the PSM information constantly while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		PsmErrorID : 		WORD; 					(*The value of the PSM error*)
	END_VAR
	VAR_IN_OUT
		Psm : 			PSM_REF; 				(*Reference to the PSM*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_PSM; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadPsmInfo_PSM
	(*DD_ReadPsmInfo: FB for PSM:
	This function block reads information concerning a PSM, like modes and certain status information.*)
	VAR_INPUT
		Enable : 						BOOL := FALSE; 			(*Get the PSM information constantly while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : 						BOOL; 					(*A valid set of outputs is available at the FB*)
		Busy : 						BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 						BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 					DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		CommunicationReady : 		BOOL; 					(*"Network" is initialized and ready for communication*)
		ReadyForPowerOn : 			BOOL; 					(*PSM is ready to be enabled (power on)*)
		PowerOn : 					BOOL;					(*If TRUE shows that the power stage is switched ON*)
		ControlMode :					DD_CONTROL_MODE_DDS; (* Indicates the current control mode *)
		InputVoltageOk :				BOOL;					(* Indicates that the UDC is ok *)
		InrushProtectionActive :			BOOL;					(* Indicates that the inrush protection is active and the device cannot be enabled. *)
		ChargeResistorProtection :		BOOL;					(* Indicates that the charge resistor protection is active and therefore the device cannot be enabled  *)
		PsmWarning : 				BOOL; 					(*Warning(s) on the PSM is present*)
		PsmError : 					BOOL;					(*Error(s) on the PSM is present*)
	END_VAR
	VAR_IN_OUT
		Psm : 						PSM_REF; 				(*Reference to the PSM*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadPsmWarning_PSM
	(*DD_ReadPsmWarning: FB for PSM:
	This function block presents general PSM warnings not relating to the function blocks *)
	VAR_INPUT
		Enable : 				BOOL := FALSE; 			(*Get the PSM information constantly while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid :				BOOL; 					(*A valid output is available at the FB*)
		Busy : 				BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 				BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 			DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		PsmWarningID : 		WORD; 					(*The value of the PSM warning*)
	END_VAR
	VAR_IN_OUT
		Psm : 				PSM_REF; 				(*Reference to the PSM*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_PSM; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadVersion_PSM
	(*DD_ReadVersion: FB for PSM:
	FB to read the version information out f the PSM*)
	VAR_INPUT
		Execute : 			BOOL := FALSE; 			(*Read the information at rising edge*)
	END_VAR
	VAR_OUTPUT
		Done : 				BOOL; 					(*A valid set of outputs is available at the FB*)
		Busy : 				BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 				BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 			DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		SerialNumber : 		STRING[18]; 				(*Serial number of the PSM*)
		MajorVersionNo : 		UINT; 					(*Major firmware number*)
		MinorVersionNo : 		UINT; 					(*Minor firmware number*)
		PatchVersionNo : UINT; (*Patch firmware number*)
		PackageLabel : STRING[20]; (*Label of the software package*)
	END_VAR
	VAR_IN_OUT
		Psm : 				PSM_REF; 				(*Reference to the PSM*)
	END_VAR
	VAR
		intern : iDD_INTERN_READ_VERS_PSM; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_Reset_PSM
	(*DD_Reset: PLCopen FB for PSM:
	This function block makes the transition from the state "Error" to "Standby" by resetting all internal PSM-related errors - 
	it does not affect the output of the FB instances.*)
	VAR_INPUT
		Execute : 		BOOL := FALSE; 			(*Resets all internal PSM-related errors*)
	END_VAR
	VAR_OUTPUT
		Done : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL;					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Psm : 			PSM_REF; 				(*Reference to the PSM*)
	END_VAR
	VAR
		intern : iDD_INTERN_RESET_DDS; (*internal variable*)
		pPsmIntern : REFERENCE TO iDD_PSM_REF_INTERN_PSM; (*internal variable of PSM*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_Trace_PSM
	(*DD_Trace: FB for PSM:
	This function block is used to start a trace in the PSM with the settings given in the input variables. After
	sending the settings and starting the trace, the FB will poll the PSM until the data has been recorded
	and afterwards uploads the data automatically.*)
	VAR_INPUT
		Execute : 				BOOL := FALSE; 									(*Starts the trace functionality at rising edge and keeps on polling*)
		Abort : 					BOOL := FALSE; 									(*Abort the on-going trace. Note that new values are only evaluated on a rising edge of Execute.*)
		pTraceBuffer : 			UDINT; 											(*Reference to a buffer where the read trace data shall be placed*)
		TraceBufferSize : 			UDINT := 0; 										(*Size of the trace buffer (size of the provided buffer given in Byte; use SIZEOF() function)*)
		SamplingRate : 			DD_SAMPLING_RATE_DDS := ddFastTask_DDS; 		(*Sampling rate of the trace*)
		SampleCount : 			UINT := 4000; 										(*Number of samples to be traced per channel*)
		SubSampling : 			UINT := 1; 										(*Multiplier to adjust time difference between trace samples*)
		SignalIDs : 				ARRAY[0..7] OF UDINT :=[ 8(0)]; 						(*Holds the IDs of the signals that should be traced*)
		TriggerID : 				UDINT := 0; 										(*ID of the signal used for triggering; Use 0 to have instantaneous tracing.*)
		TriggerPoint : 				USINT := 10; 										(*Holds the information on how many pre-trigger history is traced [in percentage]*)
		TriggerLevel : 			REAL := 0.0; 										(*Level upon which will be triggered*)
		TriggerSlope : 			BOOL := TRUE; 									(*TRUE: will trigger on rising slope; FALSE: will trigger on falling slope*)
	END_VAR
	VAR_OUTPUT
		Done : 					BOOL; 											(*The trace has successfully been recorded and read from the device.*)
		Busy : 					BOOL; 											(*The FB is not finished and new output values are to be expected*)
		CommandAborted : 		BOOL; 											(*Trace has been aborted.*)
		Error : 					BOOL; 											(*Signals that an error has occurred within the FB*)
		ErrorInfo : 				DD_ERROR_DDS;								(*Error identification and instance identifier*)
		Status : 					USINT; 											(*Holds the status of the tracing:
																				 1= Trace configured and started
																				 2= Waiting for trigger
																			  	 3= Triggered
																				 4= Uploading trace data*)
		TraceLength : 			UDINT; 											(*Length of trace data that has been uploaded*)
	END_VAR
	VAR_IN_OUT
		Psm :					PSM_REF; 										(*Reference to the PSM*)
	END_VAR
	VAR
		intern :					iDD_INTERN_TRACE_PSM; 						(*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_UpdateFirmware_PSM
	(*DD_UpdateFirmware: FB for PSM:
	This function block is used to update the appropriate firmware to a PSM. The update of the firmware is
	only allowed in Standby state. After successful file transfer, the PSM will automatically start
	updating and after that, a power-cycle is required to go back to normal operation.*)
	VAR_INPUT
		Execute : 			BOOL := FALSE; 						(*Starts the firmware update at a rising edge*)
		FileName : 			STRING[80] := ''; 						(*Filename of the firmware file on the PLC*)
		pDevice : UDINT; 		(*Pointer to the name of the PLC device on which the file is located*)
	END_VAR
	VAR_OUTPUT
		Done : 				BOOL; 								(*The firmware update process has successfully transferred the firmware to the device*)
		Busy : 				BOOL; 								(*The FB is not finished and new output values are to be expected*)
		Error : 				BOOL; 								(*Signals that an error has occurred within the FB*)
		ErrorInfo : 			DD_ERROR_DDS; 					(*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Psm : 				PSM_REF; 							(*Reference to the PSM*)
	END_VAR
	VAR
		intern : iDD_INTERN_UPDATE_FW_DDS; (*internal variable*)
		pPsmIntern : REFERENCE TO iDD_PSM_REF_INTERN_PSM; (*internal variable of PSM*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_WriteDigitalOutput_PSM
	(*DD_WriteDigitalOutput: FB for PSM:
	This function block writes a value to the digital output of the PSM. As long as "Enable" is set, the input Value is evaluated. 
	If the input "Enable" is FALSE; the last value will be used.*)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 		(*Write the value of the digital output*)
		OutputNumber : 	INT := 1; 				(*Selects the input. Value range {1; 2}*)
		Value : 			BOOL := FALSE; 		(*The value of the output (TRUE: set; FALSE: clear)*)
	END_VAR
	VAR_OUTPUT
		Valid : 			BOOL; 				(*A valid output is available at the FB*)
		Busy : 			BOOL; 				(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 				(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 	(*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Output : 			PSM_REF; 			(*Reference to the axis / signal output*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
		pPsmIntern : REFERENCE TO iDD_PSM_REF_INTERN_PSM; (*internal variable of PSM *)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_WriteParameter4_PSM
	(*DD_WriteParameter4: FB for PSM:
	FB to asynchronously write general objects of up to 4 bytes to the object dictionary of the PSM*)
	VAR_INPUT
		Execute : 		BOOL := FALSE; 			(*Write the value of the parameter at rising edge*)
		Index : 			UINT := 0; 				(*Index of the object to be written*)
		Subindex : 		USINT := 0; 				(*Subindex of the object to be written*)
		Length : 			USINT := 0; 				(*Length of the data in byte that shall be written*)
		Value : 			DWORD := 0; 				(*Value of the specified parameter*)
	END_VAR
	VAR_OUTPUT
		Done : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		AbortCode : 		DWORD; 				(*SDO abort code in case of error*)
	END_VAR
	VAR_IN_OUT
		Psm :			PSM_REF; 				(*Reference to the PSM*)
	END_VAR
	VAR
		intern : 			iDD_INTERN_SDO_WRITE_PSM; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK PSM_REF
	(*PSM_REF: Representation of the PSM for Danfoss Drive Servo with EtherCAT*)
	VAR_INPUT
		ModuleOk : BOOL; 						(*References ModuleOK from AutomationStudio*)
		NodeNumber : USINT; 					(* Node number of the device *)
		SlotName : STRING[80]; 				(*Name of the POWERLINK slot that this axis is physically connected to*)
		TPDO : ARRAY[0..8] OF UDINT; 		(* Data for IO Mapping *)
	END_VAR
	VAR_OUTPUT
		RPDO : ARRAY[0..8] OF UDINT; 		(* Data for IO Mapping *)
	END_VAR
	VAR
		intern: 						iDD_PSM_REF_INTERN_PSM; 	(* internal variable *)
	END_VAR
END_FUNCTION_BLOCK
